@@ -122,19 +122,46 @@
       "
           int count;
           struct timeval tv;
+          struct timeval tvs;
+          struct timeval tve;
+          unsigned long elapsed;
+          unsigned long remaining;
+          int retval = -1;
 
           if (#0 != Cnil) {
             tv.tv_sec = fixnnint(#0);
             tv.tv_usec = #1;
           }
-        @(return) = select(#3 + 1, (fd_set*)#2, NULL, NULL,
+          remaining = ((tv.tv_sec*1000000) + tv.tv_usec);
+
+          do {
+              (void)gettimeofday(&tvs, NULL);   // start time
+
+              retval = select(#3 + 1, (fd_set*)#2, NULL, NULL,
                            (#0 != Cnil) ? &tv : NULL);
+
+              if ( (retval < 0) && (errno == EINTR) && (#0 != Cnil) ) {
+                  (void)gettimeofday(&tve, NULL);            // end time
+                  elapsed = (tve.tv_sec - tvs.tv_sec)*1000000 + (tve.tv_usec - tvs.tv_usec);
+                  remaining = remaining - elapsed;
+                  if ( remaining < 0 ) {                     // already past timeout, just exit
+                      retval = 0;
+                      break;
+                  }
+
+                  tv.tv_sec = remaining / 1000000;
+                  tv.tv_usec = remaining - (tv.tv_sec * 1000000);
+              }
+
+          } while ((retval < 0) && (errno == EINTR));
+
+          @(return) = retval;
 " :one-liner nil)))
         (cond
           ((= 0 count)
            (values nil nil))
           ((< count 0)
-           ;; check for EINTR and EAGAIN; these should not err
+           ;; check for EAGAIN; these should not err
            (values nil (cerrno)))
           (t
            (dolist (sock sockets)
@@ -162,44 +189,58 @@
      . operation-not-permitted-error)
     (sb-bsd-sockets:protocol-not-supported-error
      . protocol-not-supported-error)
-    #-ecl
+    #-(or ecl clasp)
     (sb-bsd-sockets:unknown-protocol
      . protocol-not-supported-error)
     (sb-bsd-sockets:socket-type-not-supported-error
      . socket-type-not-supported-error)
     (sb-bsd-sockets:network-unreachable-error . network-unreachable-error)
     (sb-bsd-sockets:operation-timeout-error . timeout-error)
-    #-ecl
+    #-(or ecl clasp)
     (sb-sys:io-timeout . timeout-error)
     #+sbcl
     (sb-ext:timeout . timeout-error)
     (sb-bsd-sockets:socket-error . ,#'map-socket-error)
 
     ;; Nameservice errors: mapped to unknown-error
-    #-ecl
+    #-(or ecl clasp)
     (sb-bsd-sockets:no-recovery-error . ns-no-recovery-error)
-    #-ecl
+    #-(or ecl clasp)
     (sb-bsd-sockets:try-again-error . ns-try-again-condition)
-    #-ecl
+    #-(or ecl clasp)
     (sb-bsd-sockets:host-not-found-error . ns-host-not-found-error)))
 
-(defun handle-condition (condition &optional (socket nil))
+;; this function servers as a general template for other backends
+(defun handle-condition (condition &optional (socket nil) (host-or-ip nil))
   "Dispatch correct usocket condition."
   (typecase condition
     (serious-condition
      (let* ((usock-error (cdr (assoc (type-of condition) +sbcl-error-map+)))
-	    (usock-error (if (functionp usock-error)
-			     (funcall usock-error condition)
-			   usock-error)))
-       (when usock-error
-	 (error usock-error :socket socket))))
-    (condition (let* ((usock-cond (cdr (assoc (type-of condition)
-                                              +sbcl-condition-map+)))
-                      (usock-cond (if (functionp usock-cond)
-                                      (funcall usock-cond condition)
-                                    usock-cond)))
-                 (if usock-cond
-                     (signal usock-cond :socket socket))))))
+            (usock-error (if (functionp usock-error)
+                             (funcall usock-error condition)
+                             usock-error)))
+       (declare (type symbol usock-error))
+       (if usock-error
+           (cond ((subtypep usock-error 'ns-error)
+                  (error usock-error :socket socket :host-or-ip host-or-ip))
+                 (t
+                  (error usock-error :socket socket)))
+           (error 'unknown-error
+                  :real-error condition
+                  :socket socket))))
+    (condition
+     (let* ((usock-cond (cdr (assoc (type-of condition) +sbcl-condition-map+)))
+            (usock-cond (if (functionp usock-cond)
+                            (funcall usock-cond condition)
+                            usock-cond)))
+       (if usock-cond
+           (cond ((subtypep usock-cond 'ns-condition)
+                  (signal usock-cond :socket socket :host-or-ip host-or-ip))
+                 (t
+                  (signal usock-cond :socket socket)))
+           (signal 'unknown-condition
+                   :real-condition condition
+                   :socket socket))))))
 
 ;;; "The socket stream ends up with a bogus name as it is created before
 ;;; the socket is connected, making things harder to debug than they need
@@ -248,7 +289,7 @@ happen. Use with care."
 	  (return-from ,block ,timeout-form)))))
 
 (defun get-hosts-by-name (name)
-  (with-mapped-conditions ()
+  (with-mapped-conditions (nil name)
     (multiple-value-bind (host4 host6)
         (sb-bsd-sockets:get-host-by-name name)
       (let ((addr4 (when host4
@@ -264,7 +305,7 @@ happen. Use with care."
 		       (sockopt-tcp-nodelay-p
 			(fboundp 'sb-bsd-sockets::sockopt-tcp-nodelay)))
   (when deadline (unsupported 'deadline 'socket-connect))
-  #+ecl
+  #+(or ecl clasp)
   (when timeout (unsupported 'timeout 'socket-connect))
   (when (and nodelay-specified
              ;; 20080802: ECL added this function to its sockets
@@ -286,7 +327,7 @@ happen. Use with care."
          (socket (make-instance #+sbcl (if ipv6
                                            'sb-bsd-sockets::inet6-socket
                                            'sb-bsd-sockets:inet-socket)
-                                #+ecl 'sb-bsd-sockets:inet-socket
+                                #+(or ecl clasp) 'sb-bsd-sockets:inet-socket
                                 :type protocol
                                 :protocol (case protocol
                                             (:stream :tcp)
@@ -313,14 +354,14 @@ happen. Use with care."
                                                 (or local (host-to-vector-quad *wildcard-host*)))
                                             (or local-port *auto-port*)))
 
-              (with-mapped-conditions (usocket)
+              (with-mapped-conditions (usocket host)
 		#+(and sbcl (not win32))
 		(labels ((connect ()
                            (sb-bsd-sockets:socket-connect socket remote port)))
 		  (if timeout
 		      (%with-timeout (timeout (error 'sb-ext:timeout)) (connect))
 		      (connect)))
-		#+(or ecl (and sbcl win32))
+		#+(or ecl clasp (and sbcl win32))
 		(sb-bsd-sockets:socket-connect socket remote port)
                 ;; Now that we're connected make the stream.
                 (setf (socket-stream usocket)
@@ -375,15 +416,15 @@ happen. Use with care."
          (ip #+sbcl (if (and local (not (eq host *wildcard-host*)))
                         local
                         (hbo-to-vector-quad sb-bsd-sockets-internal::inaddr-any))
-             #+ecl (host-to-vector-quad host))
+             #+(or ecl clasp) (host-to-vector-quad host))
          (sock (make-instance #+sbcl (if ipv6
                                          'sb-bsd-sockets::inet6-socket
                                          'sb-bsd-sockets:inet-socket)
-                              #+ecl 'sb-bsd-sockets:inet-socket
+                              #+(or ecl clasp) 'sb-bsd-sockets:inet-socket
                               :type :stream
                               :protocol :tcp)))
     (handler-case
-        (with-mapped-conditions ()
+        (with-mapped-conditions (nil host)
           (setf (sb-bsd-sockets:sockopt-reuse-address sock) reuseaddress)
           (sb-bsd-sockets:socket-bind sock ip port)
           (sb-bsd-sockets:socket-listen sock backlog)
@@ -445,10 +486,19 @@ happen. Use with care."
                                "shutdown(#0, #1)" :one-liner t))
       (error (map-errno-error (cerrno))))))
 
+#+clasp
+(defmethod socket-shutdown ((usocket stream-usocket) direction)
+  (let ((sock-fd (sb-bsd-sockets:socket-file-descriptor (socket usocket)))
+        (direction-flag (ecase direction
+                          (:input 0)
+                          (:output 1))))
+    (unless (zerop (sockets-internal:shutdown sock-fd direction-flag))
+      (error (map-errno-error (cerrno))))))
+
 (defmethod socket-send ((usocket datagram-usocket) buffer size &key host port (offset 0))
   (let ((remote (when host
                   (car (get-hosts-by-name (host-to-hostname host))))))
-    (with-mapped-conditions (usocket)
+    (with-mapped-conditions (usocket host)
       (let* ((s (socket usocket))
              (dest (if (and host port) (list remote port) nil))
              (real-buffer (if (zerop offset)
@@ -456,15 +506,15 @@ happen. Use with care."
                               (subseq buffer offset (+ offset size)))))
         (sb-bsd-sockets:socket-send s real-buffer size :address dest)))))
 
-(defmethod socket-receive ((socket datagram-usocket) buffer length
+(defmethod socket-receive ((usocket datagram-usocket) buffer length
 			   &key (element-type '(unsigned-byte 8)))
   #+sbcl
   (declare (values (simple-array (unsigned-byte 8) (*)) ; buffer
 		   (integer 0)                          ; size
 		   (simple-array (unsigned-byte 8) (*)) ; host
 		   (unsigned-byte 16)))                 ; port
-  (with-mapped-conditions (socket)
-    (let ((s (socket socket)))
+  (with-mapped-conditions (usocket)
+    (let ((s (socket usocket)))
       (sb-bsd-sockets:socket-receive s buffer length :element-type element-type))))
 
 (defmethod get-local-name ((usocket usocket))
@@ -486,7 +536,7 @@ happen. Use with care."
   (nth-value 1 (get-peer-name usocket)))
 
 (defun get-host-by-address (address)
-  (with-mapped-conditions ()
+  (with-mapped-conditions (nil address)
     (sb-bsd-sockets::host-ent-name
         (sb-bsd-sockets:get-host-by-address address))))
 
@@ -525,7 +575,7 @@ happen. Use with care."
                   (sb-alien:addr rfds) nil nil
                   (when timeout secs) (when timeout musecs))))
              (if (null count) ; something wrong in #'sb-unix:unix-fast-select
-                 (unless (= err sb-unix:EINTR)
+                 (unless (= err sb-unix:eintr)
                    (error (map-errno-error err)))
                  (when (< 0 count) ; do nothing if count = 0
                    ;; process the result...
@@ -600,7 +650,7 @@ happen. Use with care."
                                               nil)))
         (ecase rv
           ((#.+wsa-wait-event-0+)
-           (update-ready-and-state-slots (wait-list-waiters wait-list)))
+           (update-ready-and-state-slots wait-list))
           ((#.+wsa-wait-timeout+)) ; do nothing here
           ((#.+wsa-wait-failed+)
            (maybe-wsa-error rv))))))
@@ -646,6 +696,11 @@ happen. Use with care."
       (boolean #.sb-vm::n-machine-word-bits)
     (event-object ws-event))
 
+  ;; not used
+  (sb-alien:define-alien-routine ("WSAResetEvent" wsa-reset-event)
+      (boolean #.sb-vm::n-machine-word-bits)
+    (event-object ws-event))
+
   (sb-alien:define-alien-routine ("WSAEnumNetworkEvents" wsa-enum-network-events)
       sb-alien:int
     (socket ws-socket)
@@ -695,13 +750,15 @@ happen. Use with care."
           (unless (zerop (ldb (byte 1 i) event-map)) ;;### could be faster with ash and logand?
             (funcall func (sb-alien:deref error-array i)))))))
 
-  (defun update-ready-and-state-slots (sockets)
-    (dolist (socket sockets)
+  (defun update-ready-and-state-slots (wait-list)
+    (loop with sockets = (wait-list-waiters wait-list)
+          for socket in sockets do
       (if (%ready-p socket)
           (progn
             (setf (state socket) :READ))
         (sb-alien:with-alien ((network-events (sb-alien:struct wsa-network-events)))
-          (let ((rv (wsa-enum-network-events (os-socket-handle socket) 0
+          (let ((rv (wsa-enum-network-events (os-socket-handle socket)
+                                             (os-wait-list-%wait wait-list)
                                              (sb-alien:addr network-events))))
             (if (zerop rv)
                 (map-network-events
@@ -751,7 +808,7 @@ happen. Use with care."
 
 ) ; progn
 
-#+(and ecl (not win32))
+#+(and (or ecl clasp) (not win32))
 (progn
   (defun wait-for-input-internal (wl &key timeout)
     (with-mapped-conditions ()
@@ -774,7 +831,7 @@ happen. Use with care."
     (declare (ignore wl w)))
 ) ; progn
 
-#+(and ecl win32 (not ecl-bytecmp))
+#+(and (or ecl clasp) win32 (not ecl-bytecmp))
 (progn
   (defun maybe-wsa-error (rv &optional syscall)
     (unless (zerop rv)
@@ -825,8 +882,9 @@ happen. Use with care."
 	(setf (state socket) :read))
       nbytes))
 
-  (defun update-ready-and-state-slots (sockets)
-    (dolist (socket sockets)
+  (defun update-ready-and-state-slots (wait-list)
+    (loop with sockets = (wait-list-waiters wait-list)
+          for socket in sockets do
       (if (%ready-p socket)
           (setf (state socket) :READ)
         (let ((events (etypecase socket
@@ -837,6 +895,7 @@ happen. Use with care."
           (multiple-value-bind (valid-p ready-p)
               (ffi:c-inline ((socket-handle socket) events) (:fixnum :fixnum)
                                                             (values :bool :bool)
+                ;; TODO: replace 0 (2nd arg) with (wait-list-%wait wait-list)
                 "WSANETWORKEVENTS network_events;
                  int i, result;
                  result = WSAEnumNetworkEvents((SOCKET)#0, 0, &network_events);
